from .widgets import ReactJSONSchemaFormWidget

try:
    # DJANGO 3.1
    from django.forms import JSONField
except ImportError:
    from django.contrib.postgres.forms.jsonb import JSONField


class ReactJSONSchemaFormField(JSONField):
    widget = ReactJSONSchemaFormWidget



from typing import List, Tuple

from jsonschema import Draft7Validator


def validate_json_schema(schema: dict) -> Tuple[bool, List[str]]:
    """
    Validate a JSON schema using the Draft 7 validator.
    """
    validator = Draft7Validator(
        schema=Draft7Validator.META_SCHEMA,
        format_checker=Draft7Validator.FORMAT_CHECKER,
    )
    errors = [f"{'.'.join(e.path)}: {e.message}" for e in validator.iter_errors(schema)]

    return not bool(errors), errors



import json
import logging

from django.forms import Media
from django.forms.widgets import Widget
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe


logger = logging.getLogger(__name__)


class ReactJSONSchemaFormWidget(Widget):

    template_name = 'django_reactive.html'

    def __init__(self, schema, ui_schema=None, on_render=None, extra_css=None, extra_js=None, **kwargs):
        self.schema = schema
        self.ui_schema = ui_schema
        self.on_render = on_render
        self.on_render_object = None
        self.extra_css = extra_css
        self.extra_js = extra_js
        super().__init__(**kwargs)

    @property
    def media(self):
        css = ['css/django_reactive.css']
        if self.extra_css:
            css.extend(self.extra_css)
        js = [
            'dist/react.js',
            'dist/react-dom.js',
            'dist/react-jsonschema-form.js',
            'js/django_reactive.js',
        ]
        if self.extra_js:
            js.extend(self.extra_js)

        return Media(css={'all': css}, js=js)

    def mutate(self):
        kwargs = {}
        if self.on_render_object:
            kwargs['instance'] = self.on_render_object
        try:
            self.on_render(self.schema, self.ui_schema, **kwargs)
        except BaseException as exc:
            logger.error('Error applying JSON schema hooks: %s', exc, exc_info=True)

    def render(self, name, value, attrs=None, renderer=None):
        if self.on_render:
            self.mutate()

        context = {
            'data': value,
            'name': name,
            'schema': json.dumps(self.schema),
            'ui_schema': json.dumps(self.ui_schema) if self.ui_schema else '{}',
        }

        return mark_safe(render_to_string(self.template_name, context))



from django.core.exceptions import ValidationError
from django.core import checks
from jsonschema import validate, ValidationError as JSONSchemaValidationError

from .widget.fields import ReactJSONSchemaFormField
from .widget.widgets import ReactJSONSchemaFormWidget
from .schema_validator import validate_json_schema

try:
    # DJANGO 3.1
    from django.db.models import JSONField as BaseJSONField
except ImportError:
    from django.contrib.postgres.fields import JSONField as BaseJSONField


class ReactJSONSchemaField(BaseJSONField):
    def __init__(self, schema=None, ui_schema=None, on_render=None, extra_css=None, extra_js=None, **kwargs):
        kwargs.setdefault('default', dict)
        super().__init__(**kwargs)
        self.schema = schema
        self.ui_schema = ui_schema
        self.on_render = on_render
        self.extra_css = extra_css
        self.extra_js = extra_js

    def formfield(self, **kwargs):
        defaults = {
            'required': not self.blank,
        }
        defaults.update(**kwargs)
        return ReactJSONSchemaFormField(
            widget=ReactJSONSchemaFormWidget(
                schema=self.schema,
                ui_schema=self.ui_schema,
                on_render=self.on_render,
                extra_css=self.extra_css,
                extra_js=self.extra_js,
            ),
            **defaults,
        )

    def validate(self, value, model_instance):
        super().validate(value, model_instance)
        try:
            validate(value, self.schema)
        except JSONSchemaValidationError:
            raise ValidationError('This field has errors.')

    def check(self, **kwargs):
        errors = super().check(**kwargs)
        res, schema_errors = validate_json_schema(self.schema)
        if not res:
            msg = ','.join(schema_errors)
            errors = [
                checks.Error(
                    f'JSON schema is not valid: {msg}',
                    obj=self.model,
                    id='fields.JSON_SCHEMA_ERROR',
                )
            ]

        return errors
